{% load i18n %}
{% load navactive %}

{% if perms.madc.basic_access %}

<li class="nav-item">
    <a class="nav-link" href="{% url 'madc:checker' character_id=character_id %}">{% translate "Doctrine Checker" %}</a>
</li>

{% endif %}
