# Generated by Django 4.2.20 on 2025-07-03 10:22

import django.core.validators
from django.db import migrations, models
import madc.models.skillchecker


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='General',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('basic_access', 'Can access this app, Voices of War.'), ('manage_access', 'Has access to moderation tools'), ('admin_access', 'Has access to all Administration tools')),
                'managed': False,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='SkillList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(default=None, max_length=500, null=True)),
                ('skill_list', models.TextField(default='', null=True, validators=[madc.models.skillchecker.skillvalidator])),
                ('ordering', models.IntegerField(default=0, validators=[django.core.validators.MinValueValidator(0)], verbose_name='Order Weight')),
                ('active', models.BooleanField(default=True, help_text='If unchecked, this skill list will not be used in the skill checker.', verbose_name='Active')),
                ('category', models.CharField(default=None, help_text='Category of the skill list, used for grouping.', max_length=100, null=True, verbose_name='Category')),
            ],
        ),
    ]
